# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2024-2025, by Samuel Williams.

source "https://rubygems.org"

gem "async-container", path: "../.."
gem "io-endpoint"

gem "puma"
gem "rack", "~> 3"
